class Styles:
    """Class defining styles for console output.

    This class provides predefined styles for different types of console output messages.
    """

    TITLE = "bold fg:ansiblue"
    SUCCESS = "fg:ansigreen"
    WARNING = "fg:ansiyellow"
    ERROR = "fg:ansired"
    INFO = "fg:ansipurple"
